@@ -1,4 +1,6 @@
 from .dagster_types import DataFrameType, MLFlowRunType, ModelType, SeriesType
+from .exporter.base import ExportComponent
+from .exporter.database import DatabaseExportComponent
 from .inference.base import InferenceComponent
 from .inference.mlflow import MLFlowInferenceComponent
 from .loader.dataframe import DataFrameLoader
