@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Sequence, Set
+from typing import Any, Dict, List, Optional, Sequence, Set
 
 from dagster import (
     Field,
@@ -44,7 +44,7 @@ class BaseComponent:
             yield Output(result, output_def.name)
 
     @classmethod
-    def to_solid(cls, name: str) -> SolidDefinition:
+    def to_solid(cls, name: str, description: Optional[str] = None) -> SolidDefinition:
         """Generates a solid definition for this component.
 
         Args:
@@ -57,6 +57,7 @@ class BaseComponent:
         """
         return SolidDefinition(
             name=name,
+            description=description,
             input_defs=cls.input_defs,
             output_defs=cls.output_defs,
             required_resource_keys=cls.required_resource_keys,
