@@ -8,17 +8,15 @@ class QDKIOManager(IOManager):
         self.values = {}
 
     def handle_output(self, context, obj):
-        keys = tuple(context.get_output_identifier())
-        self.values[keys] = obj
-
-        print("After storing:", self.values)
+        key = tuple(context.get_output_identifier())
+        self.values[key] = obj
 
         # Create a materializer for the object
         asset_materialization = Materializer(
             asset_key=[
                 context.pipeline_name,
                 context.solid_def.name,
-                keys[-1],
+                key[-1],
             ],
             object=obj,
         ).materialize()
@@ -27,12 +25,15 @@ class QDKIOManager(IOManager):
         if asset_materialization:
             yield asset_materialization
 
-        print(keys, type(obj))
-
     def load_input(self, context):
-        keys = tuple(context.upstream_output.get_output_identifier())
-        print("After loading:", self.values, keys)
-        obj = self.values[keys]
+        key = tuple(context.upstream_output.get_output_identifier())
+
+        if key not in self.values:
+            raise KeyError(
+                "The key was not found. Make sure the io_manager lives between execution processes."
+            )
+
+        obj = self.values[key]
         return obj
 
 
