@@ -1,3 +1,7 @@
+from typing import Union
+
+import dask.dataframe as dd
+import pandas as pd
 from dagster import InputDefinition, OutputDefinition
 from qdk.base import BaseComponent
 from qdk.dagster_types import DataFrameType
@@ -16,7 +20,11 @@ class TransformComponent(BaseComponent):
     ]
 
     @classmethod
-    def transform(cls, df: DataFrameType, **config) -> DataFrameType:
+    def transform(
+        cls,
+        df: Union[pd.DataFrame, dd.DataFrame],
+        **config,
+    ) -> Union[pd.DataFrame, dd.DataFrame]:
         raise NotImplementedError(
             'Make sure you added a "transform" function to the component'
         )
