@@ -10,12 +10,8 @@ from qdk.utils.text import clean_text
 
 
 class NounPhraseComponent(TransformComponent):
-    replace_pos = {
-        "ADJ": "JJ",
-        "NOUN": "NN",
-        # "PROPN": "NN",
-    }
-    noun_phrase_regex = "NP: {<NN.*|JJ>*<NN.*>}"
+    noun_phrase_regex = "NP: {<NOUN.*|ADJ>*<NOUN.*>}"
+    remove_pos = {"PUNCT"}
     config_schema = {
         "text_columns": Field(
             list,
@@ -30,15 +26,21 @@ class NounPhraseComponent(TransformComponent):
     }
 
     @classmethod
-    def extract_noun_phrases(cls, text: str, nlp: spacy.Language) -> List[str]:
+    def extract_noun_phrases(
+        cls,
+        text: str,
+        nlp: str,
+        keep_tokens: bool,
+    ) -> List[str]:
         """This function takes a string as input and outputs a list of
         noun phrase candidates:
 
-        "Dit is een zin met tekst" -> {"zin", "tekst"}
+        "Dit is een zin met tekst" -> ["zin", "tekst"]
 
         Args:
             text (str): The text you would like to transform.
-            nlp (spacy.Language): A spacy language model. Used for POS tagging.
+            nlp (str): A spacy language model. Used for POS tagging.
+            keep_tokens (bool): Whether to keep all tokens, or only noun phrases
 
         Returns:
             List[str]: The noun phrases found in the text.
@@ -47,33 +49,33 @@ class NounPhraseComponent(TransformComponent):
         cleaned_text = clean_text(text=text)
 
         # Use spacy to parse and tokenize the text
-        # Possibly replace pos tags
         doc = nlp(cleaned_text)
-        doc = [
-            (token.text.lower(), cls.replace_pos.get(token.pos_, token.pos_))
-            for token in doc
-        ]
 
-        # Use the nltk regex parser to find all noun phrases
+        # Only retrieve the text and part of speech tag
+        doc = [(token.text.lower(), token.pos_) for token in doc]
+
+        # Use the nltk regex parser to extract different parts of speech
         parser = nltk.RegexpParser(cls.noun_phrase_regex)
-        trees = parser.parse_sents([doc])
+        tree = parser.parse(doc)
 
-        # Storage for all key phrase candidates
-        noun_phrases = []
+        # Buffer for all noun phrases and extra tokens
+        tokens = []
 
-        for tree in trees:
-            for subtree in tree.subtrees(
-                filter=lambda t: t.label() == "NP"
-            ):  # For each nounphrase
-                # Concatenate the token with a space
-                noun_phrases.append(" ".join(word for word, _ in subtree.leaves()))
+        # Only select the deepest subtree, this prevents duplicate tokens
+        for subtree in tree.subtrees(lambda t: t.height() == tree.height()):
+            # The items in the subtree are either a word or a noun phrase subtree
+            for phrase in subtree:
+                # If it is a noun phrase subtree, join it and add it to the tokens buffer
+                if isinstance(phrase, nltk.tree.Tree):
+                    noun_phrases = phrase.leaves()
+                    tokens.append(" ".join(word for word, _ in noun_phrases))
+                # Otherwise keep the word if the user requests it
+                elif keep_tokens:
+                    word, pos = phrase
+                    if pos not in cls.remove_pos:
+                        tokens.append(word)
 
-        # Filter out keyphrases that have more than 5 tokens
-        noun_phrases = [
-            noun_phrase for noun_phrase in noun_phrases if len(noun_phrase.split()) <= 5
-        ]
-
-        return noun_phrases
+        return tokens
 
     @classmethod
     def transform(
@@ -81,6 +83,7 @@ class NounPhraseComponent(TransformComponent):
         df: Union[pd.DataFrame, dd.DataFrame],
         text_columns: List[str] = ["text"],
         spacy_model: str = "nl_core_news_lg",
+        keep_tokens: bool = False,
     ) -> Union[pd.DataFrame, dd.DataFrame]:
         nlp = spacy.load(
             spacy_model,
@@ -97,14 +100,14 @@ class NounPhraseComponent(TransformComponent):
             for text_column in text_columns:
                 df[f"_nouns_{text_column}"] = df.map_partitions(
                     lambda _df: _df[text_column].apply(
-                        cls.extract_noun_phrases, args=(nlp,)
+                        cls.extract_noun_phrases, args=(nlp, keep_tokens)
                     )
                 )
         # Pandas apply function
         else:
             for text_column in text_columns:
                 df[f"_nouns_{text_column}"] = df[text_column].apply(
-                    cls.extract_noun_phrases, args=(nlp,)
+                    cls.extract_noun_phrases, args=(nlp, keep_tokens)
                 )
 
         # Return the dataframe with the keywords
